using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace QuadTest
{

  public class BadGame : Microsoft.Xna.Framework.Game
  {
    const int Width = 5;
    const int Height = 5;

    GraphicsDeviceManager graphics;
    SpriteBatch spriteBatch;


    Effect quadEffect;
    RenderTarget2D renderTarget;
    DepthStencilBuffer renderDepth;
    VertexPositionTexture[] vertices;
    VertexDeclaration vertexPositionTexture;


    public BadGame()
    {
      graphics = new GraphicsDeviceManager(this);
      Content.RootDirectory = "Content";
    }


    protected override void LoadContent()
    {
      spriteBatch = new SpriteBatch(graphics.GraphicsDevice);

      quadEffect = Content.Load<Effect>("quad");
      vertexPositionTexture = new VertexDeclaration(GraphicsDevice, VertexPositionTexture.VertexElements);


      // create render target and depth buffer
      renderTarget = new RenderTarget2D(GraphicsDevice, Width, Height, 1, SurfaceFormat.Color, MultiSampleType.None, 0, RenderTargetUsage.DiscardContents);
      renderDepth = new DepthStencilBuffer(GraphicsDevice, Width, Height, DepthFormat.Depth16, MultiSampleType.None, 0);


      // set up full screen quad
      CreateFullScreenQuad();
    }


    private void CreateFullScreenQuad()
    {
      vertices = new VertexPositionTexture[4];
      vertices[0] = new VertexPositionTexture(new Vector3(-1, 1, 0f), new Vector2(0, 1));
      vertices[1] = new VertexPositionTexture(new Vector3(1, 1, 0f), new Vector2(1, 1));
      vertices[2] = new VertexPositionTexture(new Vector3(-1, -1, 0f), new Vector2(0, 0));
      vertices[3] = new VertexPositionTexture(new Vector3(1, -1, 0f), new Vector2(1, 0));
    }



    protected override void Update(GameTime gameTime)
    {
      if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed) this.Exit();
      if (Keyboard.GetState().IsKeyDown(Keys.Escape)) this.Exit();

      base.Update(gameTime);
    }


    protected override void Draw(GameTime gameTime)
    {
      // enable render target
      DepthStencilBuffer previousDepth = GraphicsDevice.DepthStencilBuffer;
      GraphicsDevice.SetRenderTarget(0, renderTarget);
      GraphicsDevice.DepthStencilBuffer = renderDepth;


      // clear the render target and set our vertex format
      GraphicsDevice.Clear(ClearOptions.Target | ClearOptions.DepthBuffer, Color.Black, 1, 0);
      GraphicsDevice.VertexDeclaration = vertexPositionTexture;


      // set up quad effect and draw
      quadEffect.Begin();

      quadEffect.Parameters["Left"].SetValue(-1.0f);
      quadEffect.Parameters["Top"].SetValue(1.0f);
      quadEffect.Parameters["Width"].SetValue(2.0f);
      quadEffect.Parameters["Height"].SetValue(2.0f);


      quadEffect.CurrentTechnique.Passes[0].Begin();
      GraphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleStrip, vertices, 0, 2);
      quadEffect.CurrentTechnique.Passes[0].End();
      quadEffect.End();


      // restore original render target and depth buffer
      GraphicsDevice.SetRenderTarget(0, null);
      GraphicsDevice.DepthStencilBuffer = previousDepth;


      // draw the quad so it looks like we're doing something
      GraphicsDevice.Clear(Color.CornflowerBlue);

      spriteBatch.Begin();
      spriteBatch.Draw(renderTarget.GetTexture(), new Rectangle(GraphicsDevice.PresentationParameters.BackBufferWidth / 2 - 64,
                                                                GraphicsDevice.PresentationParameters.BackBufferHeight / 2 - 64,
                                                                128, 128), Color.White);

      spriteBatch.End();


      base.Draw(gameTime);
    }
  }
}
